//Add event on each quantity to call price service
$(document).ready(function() {
  // Unified event listener for changes to 'currentprice', 'discount', 'quantity' in the document
  $(document).on('change', '#product-list input.currentprice, #product-list input.discount, #product-list input.quantity, #discountPercentage', function(event) {
    const row = $(this).closest('tr');
    
      // If the changed input is 'quantity', then fetch the product price
      if ($(this).attr('name') === 'quantity') {
          checkIfNumValueAndFormat($(this));
          if ($(this).val() == "0") $(this).val('');
          let quantity = parseInt($(this).val(), 10);
          const productId = $(row).find('input[name="productId"]').val();
          const customerId = $('#customerId').val();
          const today = new Date().toISOString().split('T')[0]; // Gets the current date in "YYYY-MM-DD" format
          fetchProductPrice(productId, customerId, today, quantity, row);
      }
      else{   
        checkIfNumValueAndFormat($(row).find('input.currentprice'),2);
        checkIfNumValueAndFormat($(row).find('input.discount'));
        checkIfNumValueAndFormat($('#discountPercentage'),2);
     }
     calculateNetPrice(row);
  });
});

//Clean if contains not numeric value and format to nb of decimals
function checkIfNumValueAndFormat(field,dec=0){
   var numValue=parseFloat(field.val());
   field.val(isNaN(numValue)? parseFloat("0").toFixed(dec) : numValue.toFixed(dec));
}


// Call price service
function fetchProductPrice(productId, customerId, documentDate, quantity, row) {
  
    // Disable the inputs before the fetch call
    $(row).find('input.currentprice, input.discount').prop('readonly', true).val('');
    $(row).find('td.netprice').text('');
    updateTotal();
    if (!quantity) return;
    
    const variables = {
      id: productId,
      productPrice: {
        ...(customerId && customerId !== "LEAD" && { customerId: customerId }),
        ...(customerId=="LEAD" && {documentTypeId: countriesAndDocumentTypes[$('#isoAlpha2').val()].documentTypeId}),
        documentDate: documentDate,
        quantity: quantity
      }
    };
    
    
    genericFetchPublicAPI(headerWithOrganization,qm.queryProductPrice,variables, handleResponse);
    function handleResponse(data) {
      if (data.data && data.data.productPriceById==null)
      displayBusinessWarning("PRODUCTPRICENULL","Product Price Service");
    else if (data.data && data.data.productPriceById) {
      const priceData = data.data.productPriceById;
      $(row).find('input.currentprice').val(parseFloat(priceData.price.value).toFixed(2));
      $(row).find('input.discount').val(priceData.firstDiscount.value);
    
        // Re-enable the inputs after receiving a correct response
        $(row).find('input.currentprice, input.discount').prop('readonly', false);
        calculateNetPrice(row);
        waitInfo(false);
      }
    }
  }


//Calculate net price of a row
function calculateNetPrice(row) {
    const quantity = parseFloat($(row).find('input.quantity').val() || 0);
    const currentPrice = parseFloat($(row).find('input.currentprice').val() || 0);
    const discount = parseFloat($(row).find('input.discount').val() || 0) / 100;
    
    if (quantity === 0) {
      $(row).find('.netprice').text(""); 
    } else {
      const netPrice = (currentPrice * quantity) * (1 - discount);
      $(row).find('.netprice').text(netPrice.toFixed(2)); 
    }

    $(row).find('input[name="currentprice"]').prop('readonly', (quantity == 0));
    $(row).find('input[name="discount"]').prop('readonly', (quantity == 0));

    updateTotal();
}


function resetAndDisableAllQuantitiesAndPrices(){

    var disableQuantity=!$('#customerId').val();
    $('#product-list input[name="quantity"]').not('.obsolete-row input[name="quantity"]').prop('readonly', disableQuantity).val('');
    $('#product-list input[name="name"]').not('.obsolete-row input[name="name"]').prop('readonly', disableQuantity);
    $("#code,#name,#discountPercentage").prop('readonly', disableQuantity);
    $(".trash-icon").prop('disabled', disableQuantity).toggleClass('disabled', disableQuantity);
  
    $('#product-list input[name="currentprice"]').val('');
    $('#product-list input[name="discount"]').val('');
    $('#product-list td.netprice').text('');
    $('#product-list input[name="currentprice"]').prop('readonly', true);
    $('#product-list input[name="discount"]').prop('readonly', true);
    updateTotal();
    toggleButtonsActionQuote("create");
    
}

function deleteAllProductListRows() {
    $('#product-list tr').remove();
    cleanFooter();
}

function cleanFooter(){
  $('.quoteNumber').html("...");
  $('#discountPercentage').val('');
  $('#quoteId').val('');
  $('#documentDate').val('');
  updateTotal();
  toggleButtonsActionQuote("create");
}

  
//Calculation of Total
function updateTotal() {
    let total = 0;
    let hasQuantity=false;
    
    $("#product-list tr").each(function() {
      if ($(this).attr("deleted")) return;
      let value = parseFloat($(this).find("td.netprice").text());
      if (!isNaN(value)) total += value;
      
      const quantity = $(this).find('input[name="quantity"]').val();
      if (quantity && quantity.trim() !== '') {
        hasQuantity = true;
      }
    });
  
    setButtonsActionQuoteDisabledState(!hasQuantity);

    $("#totalAmount").text(total.toFixed(2));
    var totalNetAmount = total * (1 - $("#discountPercentage").val() / 100);
    $("#totalNetAmount").text(totalNetAmount.toFixed(2));
  }
  