let countriesAndDocumentTypes = {};
let documentTypeCodes = {};

// Callback for handling document types data
function handleDocumentTypesResponse(data) {
  if (data && data.data && data.data.documentTypes) {
    const documentTypesEdges = data.data.documentTypes.edges;
    documentTypeCodes = {};
    documentTypesEdges.forEach(edge => {
      documentTypeCodes[edge.node.code] = { id: edge.node.id, code: edge.node.code };
    });
  }
}

// Callback for handling countries data
function handleCountriesResponse(data) {
  if (data && data.data && data.data.countries) {
    const countriesEdges = data.data.countries.edges;
    countriesEdges.forEach(edge => {
      const country = edge.node;
      let documentTypeIdCode = documentTypeCodes['06']; // default to 06

      if (country.isoCodeAlpha2 === activeLegislationCode) {
        documentTypeIdCode = documentTypeCodes['01'];
      } else if (country.viesCode !== null) {
        documentTypeIdCode = documentTypeCodes['02'];
      }

      countriesAndDocumentTypes[country.isoCodeAlpha2] = {
        name: country.name,
        viesCode: country.viesCode,
        documentTypeId: documentTypeIdCode ? documentTypeIdCode.id : null,
        documentTypeCode: documentTypeIdCode ? documentTypeIdCode.code : null
      };
    });
  }
}

// Mapping function
async function mapDocumentTypesToCountries() {
  await genericFetchPublicAPI(headerWithoutOrganization, qm.queryDocumentTypes, null, handleDocumentTypesResponse);
  await genericFetchPublicAPI(headerWithoutOrganization, qm.queryCountries, null, handleCountriesResponse);
  initializeCountrySelectFromMapping();
}

//Define the select 2 of countries sorted by :  1 activeLegislationCode, 2: vies Countries, 3: other countries.
function initializeCountrySelectFromMapping() {
  const selectElement = $('#isoAlpha2');
  selectElement.empty();
  const activeLegislation = [], withViesCode = [], withoutViesCode = [];

  Object.keys(countriesAndDocumentTypes).forEach(key => {
    const countryData = countriesAndDocumentTypes[key];
    if (key === activeLegislationCode) {
      activeLegislation.push({key, name: countryData.name});
    } else if (countryData.viesCode !== null) {
      withViesCode.push({key, name: countryData.name});
    } else {
      withoutViesCode.push({key, name: countryData.name});
    }
  });

  const sortByName = (a, b) => a.name.localeCompare(b.name);
  withViesCode.sort(sortByName);
  withoutViesCode.sort(sortByName);

  const addOptionsToSelect = (group, label) => {
    if (group.length > 0) {
      const optgroup = $('<optgroup>').attr('label', label);
      group.forEach(country => {
        optgroup.append(new Option(country.name, country.key));
      });
      selectElement.append(optgroup);
    }
  };

  addOptionsToSelect(activeLegislation, 'Active Legislation');
  addOptionsToSelect(withViesCode, 'Other Countries in the VIES System');
  addOptionsToSelect(withoutViesCode, 'Other Countries');

  selectElement.select2();
}

//Affect Select2 country with value coming from suggested browser addresses
$(document).ready(function() { 
    $("#autoCompleteCountry").on("change", function() {
      $("#isoAlpha2").val($(this).val()).trigger("change");
    });
});



