//Get a customer filtered by part of its name
//salesQuoteId if Get a customer from a quote
function readCustomer(nameValue,salesQuoteId){
   
    // Dynamically build the where clause
    let whereClause = '';
    const codeCondition =  `{ code: { contains: "${nameValue}" } }`;
    nameValue = nameValue.replace(/ {2,}/g, " ");
  
    if (nameValue && nameValue.split(' ').length === 2) {
        const [word1, word2] = nameValue.split(' ');
        whereClause = `
            where: { 
                or: [
                    ${codeCondition},
                    { and: [{socialName: {contains: "${word1}"}}, {socialName: {contains: "${word2}"}}]}
                ]
            }`;
    } else {
        whereClause = `
            where: { 
                or: [
                    ${codeCondition},
                    { socialName: { contains: "${nameValue}" } }
                ]
            }`;
    }
    
    genericFetchPublicAPI(headerWithOrganization,qm.queryCustomer,null, handleResponse,whereClause);
    function handleResponse(data) {
      if (data && data.data && data.data.customers && Array.isArray(data.data.customers.edges)) {
        if (data.data.customers.totalCount==0) {
          return;
        }
        $('#displayCustomerInput').prop('disabled', true);
        $('#searchCustomer').prop('disabled',false);
        const customer = data.data.customers.edges[0].node;
        $('#customerId').val(customer.id);
        $('#codeCustomer').html(customer.code);
        $('#socialNameCustomerName').html(customer.socialName);
        $('#socialNameCustomerLead').html('');
        $('#customerDiscount').val(customer.discount);
        getTariffName(customer.salesTariffId, (name) => {$('#salesTariffName').html(name);});
        getDiscountName(customer.salesDiscountGroupId, (name) => {$('#salesDiscountGroupName').html(name);});
  
        if (customer.addresses && customer.addresses.length > 0) {
          const address = customer.addresses[0];
          const formattedAddress = `
            ${address.firstLine || ""}<br>
            ${address.secondLine || ""}<br>
            ${(address.zipCode || "")}&nbsp;${address.city || ""}<br>
            ${address.countryName || ""}
        `;
          $('#addressCustomer').html(formattedAddress);
        }     
        getCustomerOrAllLastQuotes(customer.id,salesQuoteId);

        if(!salesQuoteId){
          $('#discountPercentage').val((customer.discount==0)?"":customer.discount.toFixed(2));
          resetAndDisableAllQuantitiesAndPrices();  
        }
        $('#discountPercentage').prop('readonly',false);
        $('#customerInput input').val("");
        
        waitInfo(false);
      }
    }
  }
  
  //search customer or input lead
  $(document).ready(function() {
    
    //If the "Lead Details" button is clicked, display the form to enter the Lead's information.
    //If a current quote exists, country & socialName cannot be updated
    $("#displayCustomerInput").on('click', function() {
      $("#customerInput").show();
      clearAndReadonlyCustomerInputFields();
    });

    
    // After entering the Lead's information, if 'contactName' is empty, then reset the interface; otherwise, display the Lead's details.
    $("#closeCustomerInput").on('click', function() {
      if (!$('#contactName').val()){
        resetCustomer();
      }
      else{        
        if (!$('#customerId').val()){
          $('#customerId').val("LEAD");
          $('#searchCustomer').val("");
          resetAndDisableAllQuantitiesAndPrices();
        }
        fillInCustomerInfoFromLead();

      }
      $("#customerInput").hide();
      
    }); 
    
    // If the 'contactName' is filled in, the entry of other Lead fields is allowed; otherwise, clear the form and do not permit the entry of other fields.
    $('#contactName').on('input', function() {
      clearAndReadonlyCustomerInputFields();
    });
  
    //Search for an existing customer (not a Lead) by filtering based on the entered value.
    let debounceTimeout = null;
    $('#searchCustomer').on('input', function() {
      const inputVal = this.value;
      if (!inputVal){
        resetCustomer();
        return;
      }
      clearTimeout(debounceTimeout);
      debounceTimeout = setTimeout(() => {
        if (inputVal.length > 1 || /^[0-9]/.test(inputVal.charAt(0)))  {
          readCustomer(inputVal);
        }
      }, 600);
    });

    $('#searchCustomer').on('focus', function() {
      $(this).select();
    });
  });

  //Manage clear values and readonly mode of customer input files
  function clearAndReadonlyCustomerInputFields() {
    var contactNameFilled = $('#contactName').val() !== '';
    var quoteIdFilled = $('#quoteId').val() !== '';
  
    if(!contactNameFilled && quoteIdFilled) resetCustomer();
     
    $('#customerInput input').not('#contactName').prop('readonly', !contactNameFilled);
    $('#isoAlpha2').prop('disabled',  !contactNameFilled).trigger('change');
    if (!contactNameFilled) {
      $('#customerInput input').val("");
      $('#isoAlpha2').val(activeLegislationCode).trigger('change');
    }
    else{
      $('#isoAlpha2').prop('disabled', quoteIdFilled).trigger('change');
      $('#socialName').prop('readonly', quoteIdFilled);
    }
  }


  
  //reset customer infos interface + reset all quantities
  let focusOnCustomerSearch = false;
  function resetCustomer(){
    $(".customerdetails [id]").not("button, #customerInput, select").each(function() {
      $(this).html('');
    });
    $('#displayCustomerInput').prop('disabled', false);
  
    $("#customerId").val('');
    $("#searchCustomer").val('');
    $('#customerInput input').val("");
    readProductsFromTariffList($('#templateProducts option:selected').val());
    deleteAllProductListRows();
    cleanFooter();
    getCustomerOrAllLastQuotes();
    setTimeout(() => {
      $("#searchCustomer").focus();
      focusOnCustomerSearch = true;
    }, 1000);
  }


  function fillInCustomerInfoFromLead(){
    var leadLabel=($('#customerId').val()=="LEAD")?`Lead`:"";
    $('#socialNameCustomerName').html($("#socialName").val());
    $('#socialNameCustomerLead').html(leadLabel);
    const formattedAddress = `
      ${$("#contactName").val()}<br>
      ${$("#firstLine").val()}<br>
      ${$("#secondLine").val()}<br>
      ${$("#zipCode").val()}&nbsp;${$("#city").val()}<br>
      ${$("#isoAlpha2 option:selected").text()}
    `.trim().replace(/\n/g, ' ').replace(/ {2,}/g, ' ').replace(/(<br>\s*)+/g, '<br>'); 
    $('#addressCustomer').html(formattedAddress.replace(/<br><br>/g,"<br>"));
  }

 