$('body').append(`
    <div id="businessMessageHistory" style="display:none">
        <button id="closeBusinessMessageHistory" class="buttonicon primary-action"></button>
        <button id="clearBusinessMessageHistory" class="buttonicon danger-action"></button>
        <button id="filterBusinessMessageHistory" class="buttonicon danger-action" disabled></button>
        <div id="businessMessageHistoryData"></div>
    </div>
`);

$(document).ready(function() {
    
    //Close the business message history
    $('body').on('click', '#closeBusinessMessageHistory', function() {
        hideBusinessMessageHistory();
    });
    //Clear the business message history
    $('body').on('click', '#clearBusinessMessageHistory', function() {
        clearBusinessMessageHistory();
    });
    //Filter the business message history
    $('body').on('click', '#filterBusinessMessageHistory', function() {
       filterBusinessMessageHistory();
    });
    //Access to the business message history
    $('#businessWarning, #businessError, #businessInfo,#businessMessageHistoryNb').on('click', function() {
        showBusinessMessageHistory();
    });


    //Manage the display of errors and infos
    $(document).on('focus keyup input', 'input, textarea, select,button', function(e) {
        if (e.type==="focusin"){
            if (isWaitInfoActive()) 
                setTimeout(() => {waitInfo(false);}, 1000);
        }
        else 
            hideBusinessMessages();
      });

    //Get focus to the current row in the product list table
    $(document).on('focus', 'input.quantity', function() {
     $('tr.focused').removeClass('focused');
     $(this).closest('tr').addClass('focused');
    });
    $(document).on('blur', 'input.quantity', function() {
     $(this).closest('tr').removeClass('focused');
    });
  
    // ArrowUp & ArrowDown to change row for quantity, currentprice & discount
    document.querySelector('#product-list').addEventListener('keydown', (event) => {
        if (event.target && (event.target.name === 'name' || event.target.name === 'quantity' || event.target.name === 'currentprice' || event.target.name === 'discount')) {
        let newRow = $(event.target.closest('tr'));
        let direction = null;
    
        if (event.key === 'ArrowUp') {
            direction = 'prev';
        } else if (event.key === 'ArrowDown') {
            direction = 'next';
        }
        else
        return;
        
        while (newRow) {
        newRow = newRow[direction]('tr');
            if (newRow.length==0) break; 
            
            const newInput = newRow.find(`input[name="${event.target.name}"]`);
            if (newInput.length > 0 && !newInput.prop('readonly')) {
            newInput.click();
            setTimeout(function(){newInput.focus().select();},100);
            break; 
            }
        }
        }
    });
});

//Display an error by returning the label of an error code or error code if label not found
let hasErrors=false;
function displayBusinessMessage(receivedMessageKey, context = "", messageType, forceEnableInterface = true) {
    if (!forceEnableInterface) {
        disableInterface();
    }
    hideBusinessMessages();

    console.log(`${messageType.toUpperCase()} : ENV=${sageActiveEnvironment}${context ? (" - " + context) : ""} - ${receivedMessageKey}`);
    if (context.startsWith("[HIDE]")) return;

    var message = context + ((context != "") ? " : " : "") + (errorMessages[receivedMessageKey] || receivedMessageKey);
    message = message.replace(/##ENV##/gi, "<strong>" + sageActiveEnvironment + "</strong>");

    if (messageType === "error") {
        $("#businessError").html(message).show();
        if ((receivedMessageKey == "403" || receivedMessageKey=="504" || receivedMessageKey=="530") && context == "User Profile") {
            otherEnvs({ returnToOrganizations: false, forceLogout: true });
        }
    } else if (messageType === "warning") {
        $("#businessWarning").html(message).show();
        if (receivedMessageKey == "Applicationcontext is unset." && context == "User Profile") {
            otherEnvs({ returnToOrganizations: false, forceLogout: true });
        }
    } else {
        $("#businessInfo").html(message).show();
     }

    var messageTypeCapitalized = messageType.charAt(0).toUpperCase() + messageType.slice(1).toLowerCase();
    var newMessageContent = `<div class="business${messageTypeCapitalized}">${message}</div>`;
    var childrenCount = $("#businessMessageHistoryData").children().length;
    $("#businessMessageHistoryNb").text(childrenCount+1);
    $("#businessMessageHistoryData").append(newMessageContent);
    
    if (messageType === 'error') {
        disableFilterBusinessMessageHistory(false);
    }
    if (isFilteredErrorMessages() && messageType !== 'error') $("#businessMessageHistoryData").children().last().hide();
    
    if (childrenCount > 0) $("#businessMessageHistoryNb").show();

    $("#businessMessageHistory").scrollTop($("#businessMessageHistory")[0].scrollHeight);
    if (messageType === 'warning') {
        if (!$("#closeBusinessMessageHistory").hasClass("closeBusinessMessageHistoryWarning") && !$("#closeBusinessMessageHistory").hasClass("closeBusinessMessageHistoryError")) {
            $("#closeBusinessMessageHistory").addClass("closeBusinessMessageHistoryWarning").removeClass("closeBusinessMessageHistoryInfo");
        }
        if (!$("#businessMessageHistoryNb").hasClass("businessMessageHistoryNbWarning") && !$("#businessMessageHistoryNb").hasClass("businessMessageHistoryNbError")) {
            $("#businessMessageHistoryNb").addClass("businessMessageHistoryNbWarning").removeClass("businessMessageHistoryNbInfo");
        }
    } else if (messageType === 'error') {
        if (!$("#closeBusinessMessageHistory").hasClass("closeBusinessMessageHistoryError")) {
            $("#closeBusinessMessageHistory").removeClass("closeBusinessMessageHistoryWarning closeBusinessMessageHistoryInfo").addClass("closeBusinessMessageHistoryError");
        }
        if (!$("#businessMessageHistoryNb").hasClass("businessMessageHistoryError")) {
            $("#businessMessageHistoryNb").addClass("businessMessageHistoryNbError").removeClass("businessMessageHistoryNbInfo businessMessageHistoryNbWarning");
        }
    }


    if (forceEnableInterface) {
        waitInfo(false);
        enableInterface();
    }
}

function displayBusinessError(receivedErrorKey, context = "", forceEnableInterface = true) {
    displayBusinessMessage(receivedErrorKey, context, "error", forceEnableInterface);
}

function displayBusinessWarning(receivedWarningKey, context = "", forceEnableInterface = true) {
    displayBusinessMessage(receivedWarningKey, context, "warning", forceEnableInterface);
}
function displayBusinessInfo(info,forceEnableInterface=true) {
    displayBusinessMessage(info, "", "info", forceEnableInterface);
}

function hideBusinessMessages(){
   tooLongInfo=false;
   $("#businessInfo").text("").hide();
   $("#businessError").text("").hide();
   $("#businessWarning").text("").hide();
}

function hideBusinessMessageHistory() {
    $("#businessMessageHistory").hide();
}

function showBusinessMessageHistory() {
    $("#businessMessageHistory").show();
    $("#businessMessageHistory").scrollTop($("#businessMessageHistory")[0].scrollHeight);
}

function clearBusinessMessageHistory() {
    $("#filterBusinessMessageHistory").removeClass("isFilterBusinessMessageHistory");
    disableFilterBusinessMessageHistory();
    $("#closeBusinessMessageHistory").removeClass("closeBusinessMessageHistoryWarning closeBusinessMessageHistoryError").addClass("closeBusinessMessageHistoryInfo");
    $("#businessMessageHistoryNb").removeClass("businessMessageHistoryNbWarning businessMessageHistoryNbError").addClass("businessMessageHistoryNbInfo");
    $("#businessMessageHistoryData").html('');
    $("#businessMessageHistoryNb").hide();
    hideBusinessMessages();
}

function disableFilterBusinessMessageHistory(state=true){
    $("#filterBusinessMessageHistory").prop("disabled", state);
}   

function filterBusinessMessageHistory() {
    if (isFilteredErrorMessages()) {
        $("#businessMessageHistoryData").children().show();
        $("#filterBusinessMessageHistory").removeClass("isFilterBusinessMessageHistory");
    } else {
        $("#businessMessageHistoryData").children().hide();
        $("#businessMessageHistoryData").children(".businessError").show();
        $("#filterBusinessMessageHistory").addClass("isFilterBusinessMessageHistory");
    }
}

function isFilteredErrorMessages() {
    return $("#filterBusinessMessageHistory").hasClass("isFilterBusinessMessageHistory");
}

const errorMessages = {
"global.businessErrors.authorizationPolicy": "The current user does not have the necessary policy permissions to perform this action",
"sales.businessErrors.invalidCode": "The product code is invalid",
"sales.businessErrors.invalidName": "The product name is invalid",
"sales.businessErrors.codeAlreadyExistsInOrganization": "The product code is already in use",
"accounting.businessErrors.accountingExerciseNotFound": "You must create a fiscal year for the current year in order to create quotes",
"sales.businessErrors.cannotDeleteSalesOfferRelatedToSalesOrders":"Quote cannot be deleted because is linked to at least one Order",
"sales.businessErrors.salesOfferCannotBeDeletedBecauseIsLinkedToAtLeastOneSalesOrder":"Quote cannot be deleted because is linked to at least one Order",
"Cannot accept null for non-nullable input": " ",
"NODATAFOUND": "No item was found",
"PRODUCTPRICENULL": "The price service was unable to calculate the price",
"401": "Unauthorized.  Please log out and restart the application to authenticate again.",
"400": `Bad request. Are you sure that environment ##ENV## has the latest features of the public API?`,
"403": `Forbidden. Your user is not authorized to use this app for environment ##ENV##. Change the environment from the list or click Logout and follow the instructions.`,
"404": `Not found. The public API is not available for environment ##ENV##`,
"Applicationcontext is unset.": `Your user has been blocked in environment ##ENV##. Therefore, you cannot use this application for environment ##ENV##.`,
"500": `Internal server error. Try again by refreshing this page`,
"504": `Gateway timeout for environment ##ENV##. Try again by refreshing this page or change the environment if ##ENV## is wrong.`,
"530": "The environment ##ENV## appears to be inaccessible. Try again by refreshing this page or change the environment if ##ENV## is wrong.",
"Invalid JWT.":"The access token is invalid. Please log out and restart the application to authenticate again.",
"Failed to fetch":"Failure of the public API fetch through the proxy, check if the proxy is operational.",
"UNKNOWNUSERPOLICY": "The user policy is unknown",
"APPFORBIDDEN":"The current user's roles do not authorize access to the quotes in the current business, therefore, this application is not usable for the current user and the current business.",
"ALLNOTREADY": "No business is ready to be used.",
"NOTREADY": "This business is not ready to be used."
};













