
//Get the id list of products defined in a tariff, if not found get the full list of products
async function readProductsFromTariffList(code) {
  var tariffId;
  var tariffName;

  if (!code) {
    for (const [id, tariffInfo] of Object.entries(tariffsDictionary)) {
      if (tariffInfo.template === true) {
        code = tariffInfo.code;
        break; 
      }
    }
  }
 

  for (const [id, tariffInfo] of Object.entries(tariffsDictionary)) {
    if (tariffInfo.code === code) {
      tariffId=id;
      tariffName=tariffInfo.name;
      if (tariffId.startsWith("FAKE_")) {
        await fake_tariff(code);
        return;
      }
      break;
    }
  }
  if (!tariffId){
    populateTariffDropdown();
    $('#templateProducts').val();
    toggleButtonsActionQuote("create");
    readProducts();
    return;
  }

  const variables = { id: tariffId };
  await genericFetchPublicAPI(headerWithOrganization, qm.queryTariffProducts, variables,handleResponse);
  function handleResponse(data){  
    var infoLines;
    if (data && data.data && data.data.salesTariffs && Array.isArray(data.data.salesTariffs.edges)) {
      infoLines = data.data.salesTariffs.edges[0].node.lines;
      populateTariffDropdown();
      $('#templateProducts').val(code);
      toggleButtonsActionQuote("create");
      readProducts(infoLines);
    }
  }
}

// if fake tariff, get the list of all products filtered to return some products to emulate a template
async function fake_tariff(code) {
  let whereClause = '';
  if (code === "BESTSALES") {
      whereClause = `where: { code: { contains:"RING" } }`;
  } else if (code === "FAVORITES") {
      whereClause = `where: {
        or: [
          { code: { startsWith: "GLD" } },
          { code: { startsWith: "SILV" } }
        ]
      }`;
  }
  var qmFake={...qm.queryProducts};
  qmFake.noDataMessage="No products found in the demonstration template "+code;

  await genericFetchPublicAPI(headerWithOrganization, qmFake, null,handleResponse,whereClause);
  function handleResponse(data){  
    var infoLines;
    if (data && data.data && data.data.products && Array.isArray(data.data.products.edges) && data.data.products.edges.length > 0) {
      infoLines = data.data.products.edges.map(edge => {
        const node = edge.node;
        return {
            ...node, 
            productId: node.id
        };
      });
      toggleButtonsActionQuote("create");
      readProducts(infoLines);
    }
    populateTariffDropdown();
    $('#templateProducts').val(code);
  }
}



// Get the list of all products or products from template or quote
function readProducts(infoLines=""){

    if (infoLines){
      const productIds = infoLines.map(line => `"${line.productId}"`);
      var whereClause = (productIds)?`where: { id: { in: [${productIds.join(", ")}] } }`:"";   
    }
    else {
      whereClause="";
    }

    genericFetchPublicAPI(headerWithOrganization,qm.queryProducts,null, handleResponse,whereClause);
    
    function handleResponse(data) {
      if (data && data.data && data.data.products && Array.isArray(data.data.products.edges)) {
        const productList = data.data.products.edges;
        const productDetails={};

        productList.forEach(product => {
          productDetails[product.node.id] = product.node;
        });
        
        $('#createsearch').remove();
        $('tbody#product-list tr').remove();

        if (infoLines === "") {
          productList.forEach(product => {
              const productRow = createProductRow(product.node);
              productRow.click(handleProductRowClick(product.node));
              $("#product-list").append(productRow);
          });
        }
        else{
          infoLines.forEach(line => {
            const productInfo = productDetails[line.productId];
            if (productInfo) {
              const productRow = createProductRow(productInfo, line); 
              productRow.click(handleProductRowClick(productInfo)); 
              $("#product-list").append(productRow);
              calculateNetPrice(productRow);
            }
          });
        }
  
        $("table.productlist").css("min-height", "unset");
        $("#product-list tr").eq(0).click();
        addCreateOrSearchProductInputRow();
        if (!infoLines || !infoLines.some(line => line.hasOwnProperty('totalQuantity'))) {
          resetAndDisableAllQuantitiesAndPrices();
        }        
        enableInterface();
        waitInfo(false);

      }
    }
  }
  
  // query to search a product by Code or Name
  function searchProductByCodeOrName(confirmInsert) {
    const codeValue = $('#code').val();
    const nameValue = $('#name').val();
  
    let whereClause = '';
  
    if (codeValue) {
        whereClause = `where: { code: { contains: "${codeValue}" } }`;
    } else if (nameValue) {
        const nameParts = nameValue.split(' ');
    
        if (nameParts.length === 2) {
            const [word1, word2] = nameParts;
            whereClause = `
            where: { and: [{name: {contains: "${word1}"}},{name: {contains: "${word2}" } }] }`; 
        } else {
            whereClause = `where: { name: { contains: "${nameValue}" } }`;
        }
    }
  
    genericFetchPublicAPI(headerWithOrganization, qm.queryProducts, null, handleResponse,whereClause);
  
    function handleResponse(data) {
      if (data && data.data && data.data.products && Array.isArray(data.data.products.edges) && data.data.products.edges.length > 0) {
        const product = data.data.products.edges[0].node;
        if (confirmInsert){
            $('#createsearch').remove();
            const productRow = createProductRow(product);
            productRow.click(handleProductRowClick(product));
            $("#product-list").prepend(productRow);
            addCreateOrSearchProductInputRow();
            productRow.click();
            waitInfo(false);
        }
        else {
         displayBusinessInfo(product.code+" : "+product.name);
        }
      }
    }
  }

  
//Search product, activation of treatment if minimum of 2 chars filled in for code or name
//createproduct enabled button if code & name filled in
$(document).ready(function() {



    let debounceTimeout = null;
    function checkInputs() {
      clearTimeout(debounceTimeout);
      debounceTimeout = setTimeout(() => {

      adjustButtonAccessibility("createproduct",userPolicies.createProduct, !$('#code').val() || !$('#name').val() );

      var search=($('#code').val().length>=2 || $('#name').val().length>=2);
      adjustButtonAccessibility("searchproduct",userPolicies.products,!search );
       if (search) searchProductByCodeOrName();
      },600);
    }
    $(document).on('keyup input', '#createsearch input.code, #createsearch input.name', checkInputs);
  });
    
   
    $(document).ready(function() {
    // Event listener for changes to 'salesUnitPrice' & 'currentprice' in the document
    $(document).on('change', '#createsearch input.salesUnitPrice,#product-list input.currentprice,#discountPercentage', function(event) {
      checkIfNumValueAndFormat($(this),2);
    });
  });
  
  
  
  


// mutation to add a new product
function createNewProduct() {
    const salesUnitPriceValue = $("#salesUnitPrice").val();
    const values = {
      "values": {
        "code": $("#code").val().toUpperCase(),
        "name": $("#name").val(),
        "category": $("#category option:selected").val(),
        "salesUnitPrice":(salesUnitPriceValue!="" && !isNaN(parseFloat(salesUnitPriceValue))) ? parseFloat(salesUnitPriceValue) : 0
      }
    };
  
    genericFetchPublicAPI(headerWithOrganization,qm.createProduct,values, handleResponse);
    function handleResponse(data) {
     //Add a new line in list of products with info of the created product (retrieve product id from API response)
     if (data && data.data && data.data.createProduct) {
          const product = {
            id: data.data.createProduct.id,
            code:  $("#code").val().toUpperCase(),
            name: $("#name").val(),
            category: $("#category").val(),
            salesUnitPrice: ($("#salesUnitPrice").val()!="" && !isNaN(parseFloat($("#salesUnitPrice").val()))) ? $("#salesUnitPrice").val() : "0",
            obsolete: false  
          };
          $('#createsearch').remove();
          const productRow = createProductRow(product);
          productRow.click(handleProductRowClick(product));
          $("#product-list").prepend(productRow);      
          productRow.click();
          addCreateOrSearchProductInputRow();
          waitInfo(false);
      }
    }
  }
  
  // Dynamically generate a row of product list table
  function createProductRow(product,line="") {
    const productId = product.id || product.node?.id;
    const lineId=line.id || "";
    const isObsolete = product.obsolete || product.node?.obsolete;
    const code = product.code || product.node?.code;
    const name = line.productName || product.name || product.node?.name;
    const categoryValue = product.category || product.node?.category;
    const category = categoryValue.charAt(0).toUpperCase() + categoryValue.toLowerCase().slice(1); 
    const salesUnitPrice = parseFloat(product.salesUnitPrice !== undefined ? product.salesUnitPrice : product.node?.salesUnitPrice).toFixed(2);
    const salesVatPercentage = product.salesVatPercentage !== undefined ? product.salesVatPercentage : product.node?.salesVatPercentage;
    const quantity=line.totalQuantity || "";
    const currentprice = line.unitPrice ? parseFloat(line.unitPrice).toFixed(2) : "";
    const discount=line.firstDiscount || "";

    return $(`
    <tr${isObsolete ? ' class="obsolete-row"' : ''}>
      <td class="code">${code}
        <input name="lineId" type=hidden value="${lineId}">
        <input name="productId" type=hidden value="${productId}">
        <input name="salesVatPercentage" type=hidden value="${salesVatPercentage}">
      </td>
     
      <td class="name"><input name="name" value="${name}" class="name" type="text" ${isObsolete ? 'readonly' : ''} autocomplete="off"/></td>
      <td class="category">${category}</td>
      <td class="price">${salesUnitPrice}</td>
      <td class="quantity"><input name="quantity" value="${quantity}" class="quantity" type="text" ${isObsolete ? 'readonly' : ''} autocomplete="off"/></td>
      <td class="currentprice"><input name="currentprice" value="${currentprice}" class="currentprice" type="text" readonly autocomplete="off"/></td>
      <td class="discount"><input name="discount" value="${discount}" class="discount" type="text" readonly autocomplete="off"/></td>
      <td class="netprice"></td>
      <td class="delete"><span class="trash-icon"></span></td>
    </tr>`
    ).on('click', '.trash-icon', function() {
      clickOnTrash(this);
    });
  }

  function clickOnTrash(line) {
    const row = $(line).closest('tr');
    const allRows = $('tr').not('.createsearch');
    const rowIndex = allRows.index(row);

    row.attr('deleted', true).css('display', 'none');
    updateTotal();

    var otherRow = findNextVisibleRow(allRows, rowIndex);
    if (otherRow.length > 0) 
        setTimeout(function() { otherRow.trigger('click'); }, 0);
    else
        setTimeout(function() { $('.productdetails [id]').empty(); }, 0);
}

function findNextVisibleRow(allRows, index) {
    for (let i = index + 1; i < allRows.length; i++) {
        if ($(allRows[i]).css('display') !== 'none') return $(allRows[i]);
    }
    for (let i = index - 1; i >= 0; i--) {
        if ($(allRows[i]).css('display') !== 'none') return $(allRows[i]);
    }
    return $();
}

  
// function called on click on a product row in the table
function handleProductRowClick(product) {
  return function() {
    const formattedLineDescription = product.lineDescription ? product.lineDescription.replace(/\n/g, '<br>') : '';
    const formattedComments = product.comments ? product.comments.replace(/\n/g, '<br>') : '';
  
    $('#nameProduct').html(product.name);
    $('#lineDescription').html(formattedLineDescription);
    $('#comments').html(formattedComments);

    var isActiveElementPriceOrDiscountNotReadonly = $(document.activeElement).is('input.name:not([readonly]),input.currentprice:not([readonly]), input.discount:not([readonly])');
    if (!isActiveElementPriceOrDiscountNotReadonly && !focusOnCustomerSearch) {
      $(this).find('input.quantity').focus().select();
      focusOnCustomerSearch=false;
    }
  }
}




  // Add a new row to allow to add a product
function addCreateOrSearchProductInputRow() {
    const inputRow = $(`
      <tr class="createsearch" id="createsearch">
        <td><input id="code" class="code" type="search" autocomplete="off"></td>
        <td><input id="name" class="name" type="search" autocomplete="off"></td>
        <td>
          <select id="category" class="category" disabled>
            <option value="PRODUCT">Product</option>
            <option value="SERVICE">Service</option>
          </select>
        </td>
        <td class="salesUnitPrice"><input id="salesUnitPrice" class="salesUnitPrice" type="text" disabled autocomplete="off"></td>
        <td class=quantity colspan=5>
          <button id=createproduct class="createsearchproduct primary-action" onclick="createNewProduct()" disabled>Create and insert</button>
          <button id=searchproduct class="createsearchproduct primary-action" onclick="searchProductByCodeOrName(true)" disabled>Search and insert</button>
        </td>
      </tr>
    `);
    $("#theadProductlist").append(inputRow);

    adjustButtonAccessibility("createproduct",userPolicies.createProduct,true);
    adjustButtonAccessibility("searchproduct",userPolicies.products,true );
  
      // Event listeners to handle changes in 'code' and 'name' fields
      inputRow.find('.code, .name').on('keyup input', function() {
        inputRow.find('.category, .salesUnitPrice').prop('disabled', !inputRow.find('.code').val() || !inputRow.find('.name').val());
      });
  }
  
  