const qm = {

  queryUserProfile: {
    context: "User Profile",
    policy: "userProfile",
    body: `
      query {
        userProfile {
          applicationLanguageCode
          authenticationEmail
          fullName
        }
      }
      `
  },

  queryOrganizations: {
    context: "Organizations",
    policy: "organizations",
    body: `
      query  {
        organizations(
            first:500
            order: [{ creationDate: DESC }]
        ) {
            edges {
                node {
                  id
                  tenantId
                  socialName
                  legislationCode
                  creationDate
                  onboardingCompleted
                }
            }     
        }
      }
      `
  },

  queryOrganizationAccountingSetupByOrgId:{
    context: "Organization Accounting Setup",
    policy: "organizationAccountingSetupByOrgId",
    body: `
      query ($id:ID!){
        organizationAccountingSetupByOrgId(id: $id) {
            id
            organizationId 
            defaultCustomersAccountId
            defaultSuppliersAccountId
            subAccountingLength
            taxRegime
        }
     }
      `
  },

  queryUserAccessPolicyCheck: {
    context: "User policies check",
    policy: "none",
    body: `
      query ($actions: [String!]!) {
        userAccessPolicyCheck (actions: $actions) {
          action
          isAllowed
        }     
      }
      `
  },




  queryCountries: {
    context: "Countries",
    policy: "countries",
    body: `
      query {
        countries(
          first: 500
          order: {name:ASC}
        ) {
          edges {
            node {
              isoCodeAlpha2
              name
              viesCode
            }
          }
        }
      }
      `
  },

  queryDocumentTypes: {
    context: "Document Types",
    policy: "documentTypes",
    body: `
      query {
        documentTypes {
            edges {
                node {
                    id
                    code
                }
            }
        }    
      }
      `
  },

  queryCustomer: {
    context: "Customer",
    policy: "customers",
    body: `
      query {
        customers(
          first: 1
          order: { code: ASC }
          %%WHERECLAUSE%%
          ) {
            edges {
              node {
                id
                code
                socialName   
                salesTariffId   
                salesDiscountGroupId
                hasEquivalenceSurcharge 
                discount
                addresses {
                  firstLine
                  secondLine
                  city
                  zipCode
                  countryName        
              }       
              }
            }
            totalCount  
          }  
      }
      `
  },

  queryTariffProducts: {
    context: "Tariff Products",
    policy: "salesTariffs",
    body: `
      query  ($id: UUID!) {
        salesTariffs 
        (
            where: {id: {eq:$id}}
        ) {
          edges {
            node {
              name
              lines {
                productId
              }
            }
          }
        }
      }
      `
  },

  queryTariffById: {
    context: "Tariff",
    policy: "salesTariffs",
    body: `
      query  ($id: UUID!) {
        salesTariffs 
        (
            where: {id: {eq:$id}}
        ) {
          edges {
            node {
              name
            }
          }
        }
      }
      `
  },

  queryDiscountById: {
    context: "Discount",
    policy: "salesDiscounts",
    body: `
      query  ($id: UUID!) {
        salesDiscounts 
        (
            where: {id: {eq:$id}}
        ) {
          edges {
            node {
              name
            }
          }
        }
      }
      `
  },

  queryTariffs: {
    context: "Tariffs",
    policy: "salesTariffs",
    body: `
      query {
        salesTariffs 
        (first:500) {
          edges {
            node {
              id
              code
              name
              enabled
              startDate
              endDate
            }
          }
          totalCount
        }
      }
      `
  },

  queryProducts: {
    context: "Products",
    policy: "products",
    body: `
      query  {
        products (
          first:100
          order: { code: ASC }
          %%WHERECLAUSE%%
          ){
            edges {
              node {
                id
                code
                comments
                creationDate
                firstSalesDiscount
                taxGroupId
                lineDescription
                name
                obsolete
                salesUnitPrice
                salesVatPercentage
                category
                id
              }
            }
            totalCount
            pageInfo {
              hasNextPage
            }
          }     
        }
      `
  },

  createProduct: {
    context: "Product Creation",
    policy: "createProduct",
    body: `
      mutation ($values: ProductCreateGLDtoInput!) {  
        createProduct(input: $values) {
          id
        }
      }
      `
  },

  queryProductPrice: {
    context: "Product Price Service",
    policy: "productPriceById",
    body: `
      query($id: ID!, $productPrice: ProductPriceGLDtoInput!) {
        productPriceById(id: $id, productPrice: $productPrice) {
          equivalenceSurchargePercentage
          firstDiscount {
            code
            name
            source
            value
          }
          price {
            code
            name
            source
            value
          }
          taxGroupId
          taxId
          taxPercentage
          taxTreatmentId
        }
      }
      `
  },

  createSalesQuote: {
    context: "Sales Quote Creation",
    policy: "createSalesQuote",
    body: `
      mutation ($values: SalesQuoteCreateGLDtoInput!) {  
        createSalesQuote(input: $values) {
          id
        }
      }
      `
  },

  updateSalesQuote: {
    context: "Sales Quote Update",
    policy: "updateSalesQuote",
    body: `
      mutation ($values: SalesQuoteUpdateGLDtoInput!) {  
        updateSalesQuote (input: $values) {
          id
        }
      }
      `
  },

  deleteSalesQuote: {
    context: "Sales Quote Deletion",
    policy: "deleteSalesQuote",
    body: `
      mutation ($values:SalesQuoteDeleteGLDtoInput!) {
        deleteSalesQuote(input: $values) {
        id
        }
      }
      `
  },

  queryQuotesOfCurrentCustomerOrAll: {
    context: "Quotes",
    policy: "salesQuotes",
    body: `
      query {
        salesQuotes (
              first:100
              order: { operationalNumber: DESC }
              %%WHERECLAUSE%%
          )  {
              edges {
                  node {
                      id
                      socialName
                      operationalNumber
                      creationDate
                      customerId
                      documentDate
                      status
                      totalNet
                  }
              }
              totalCount
              pageInfo {
              hasNextPage
              }
          }
      }
      `
  },

  queryQuoteById: {
    context: "Quote",
    policy: "salesQuotes",
    body: `
      query ($id: UUID!) {
        salesQuotes (
              where: {id: {eq:$id}}
          )  {
              edges {
                  node {
                      documentDate
                      operationalNumber
                      status
                      customerId
                      totalCustomerDiscount
                      totalDiscount
                      totalFeeSurcharge
                      totalGross
                      totalLiquid
                      totalNet
                      totalVat
                      totalVatBase
                      totalVatFee
                      discount
                      lines  
                      ( order: { creationDate: ASC }){
                          id
                          pendingQuantity
                          productId
                          productName
                          status
                          totalNet
                          totalQuantity
                          unitPrice
                          firstDiscount
                          vatPercentage            
                      }
                      countryAcronym
                      socialName
                      contactName
                      contactPhone
                      contactEmail
                      mainAddress{
                        firstLine
                        secondLine
                        zipCode
                        city
                        countryName
                      }
                  }
              }
          }
      }
      `
  },

  queryGetPdf: {
    context: "Get PDF",
    policy: "documentPdfPreviewSalesQuote",
    body: `
      query($input: DocumentPdfPreviewGLDtoInput!) {
        documentPdfPreview(input: $input) {
          type
          id
          pdfPreview
        }
      }
      `
  },

  createAccountingEntry: {
    context: "Accounting Entry Creation",
    policy: "createAccountingEntryUsingCodes",
    body: `
      mutation CreateAccountingEntryUsingCodes ($values:AccountingEntryCreateUsingCodesGLDtoInput!) {    
        createAccountingEntryUsingCodes (input: $values) {
          id
        }
      }    
      `
  },

  createCustomer: {
    context: "Customer Creation",
    policy: "createCustomer",
    body: `
      mutation ($values: CustomerCreateGLDtoInput!) {  
        createCustomer (input: $values) {
          id
        }
      }       
      `
  },

  createSupplier  : {
    context: "Supplier Creation",
    policy: "createSupplier",
    body: `
      mutation ($values: SupplierCreateGLDtoInput!) {  
        createSupplier (input: $values) {
          id
        }
      }       
      `
  },
  createEmployee  : {
    context: "Employee Creation",
    policy: "createEmployee",
    body: `
      mutation ($values: EmployeeCreateGLDtoInput!) {  
        createEmployee (input: $values) {
          id
        }
      }       
      `
  },

  importAccountingPlanMaster  : {
    context: "Accounting Plan Master Import",
    policy: "accountingPlanMasterImport",
    body: `
      mutation ($values: AccountingPlanMasterImportGLDtoInput!) {  
          accountingPlanMasterImport(input: $values) {
          id
        }
      } 
    `
  },

  createAccountingExercise  : {
    context: "AccountingExercise Creation",
    policy: "createAccountingExercise",
    body: `
      mutation ($values: AccountingExerciseCreateGLDtoInput!) {  
        createAccountingExercise (input: $values) {
          id
        }
      }    
      `
  },

  createSalesQuote:{
    context: "Sales Quote Creation",
    policy: "createSalesQuote",
    body: `
      mutation ($values: SalesQuoteCreateGLDtoInput!) {  
        createSalesQuote(input: $values) {
          id
        }
      }
    `
  },

  createSalesOrder:{
    context: "Sales Order Creation",
    policy: "createSalesOrder",
    body: `
      mutation ($values: SalesOrderCreateGLDtoInput!) {  
        createSalesOrder(input: $values) {
          id
        }
      }
    `
  },

  createSalesDeliveryNote: {
    context: "Sales Delivery Note Creation",
    policy: "createSalesDeliveryNote",
    body: `
      mutation ($values: SalesDeliveryNoteCreateGLDtoInput!) {  
        createSalesDeliveryNote(input: $values) {
          id
        }
      }
    `
  },

  createSalesInvoice: {
    context: "Sales Invoice Creation",
    policy: "createSalesInvoice",
    body: `
      mutation ($values: SalesInvoiceCreateGLDtoInput!) {  
        createSalesInvoice(input: $values) {
          id
        }
      }
      `
  },
  
 closeSalesInvoice: {
    context: "Close Sales Invoice",
    policy: "createSalesInvoice",
    body: `
      mutation($input: CloseSalesInvoiceGLDtoInput!) {
        closeSalesInvoice(input: $input) {
        id
        operationalNumber
      }
    }
    `
  },
  
 postSalesInvoice: {
    context: "Post Sales Invoice",
    policy: "createSalesInvoice",
    body: `
      mutation($input: PostSalesInvoiceGLDtoInput!) {
        postSalesInvoice(input: $input) {
          accountingEntryId
          accountingEntryNumber
      }
    }
    `
  },

 querySalesJournalId: {
    context: "Get Sales Journal Id",
    policy: "journalTypes",
    body: `
     query {
              journalTypes(
                  where: { type: { eq: SALES_INVOICE } }
                  first: 1
              ) {
                  edges {
                      node {
                          id
                      }
                  }
              }
          }
    `
  },

  queryCheckOrganizationData :{
    context: "Check Organization Demo Data",
    policy: "none",
    body: `
    query {
          accountingExercises(order: [{ startDate: ASC }]) {edges {node {id startDate}}},
          salesInvoices(first: 1, where: {salesOrderNumber: {eq:"IMPORTDEMO"}} ) {edges {node {id}}},
          accountingEntries(first: 1) {edges {node {id}}},
          customers(first: 1,where: {tradeName: {eq: "Diamant Vert"}}) {edges {node {code}}},
          accountingAccounts(first: 1) {edges {node {id}}},
          products(first: 1, where: {code: {eq: "GLDCHAIN"}}) {edges {node {code}}}
        }     
    `
  },

  queryCustomersIdByCode: {
    context: "Customer",
    policy: "customers",
    body: `
     query {
        customers(%%WHERECLAUSE%%) {
            edges {
              node {
                id
                code
              }
            }
        }
      }
    `
  },
  queryProductsIdByCode: {
    context: "Products",
    policy: "products",
    body: `
     query {  
       products(%%WHERECLAUSE%%) {
            edges {
              node {
                id
                code
              }
            }
        } 
      }
    `
  }
};

