function toggleButtonsActionQuote(actionType) {
  var $createQuote = $('.createQuote');
  var $updateQuote = $('.updateQuote');
  var $duplicateQuote = $('.duplicateQuote');
  var $updateCreateQuote = $('.updateCreateQuote');

  switch (actionType) {
      case 'create':
          $createQuote.show();
          $updateCreateQuote.hide();
          $(".quoteNumber").html("New quote");
          break;
      case 'updateOrDuplicate':
          $createQuote.hide();
          $updateCreateQuote.show();
          $updateQuote.removeClass('secondary-action').addClass('primary-action');
          $duplicateQuote.removeClass('primary-action').addClass('secondary-action');
          adjustButtonAccessibility('updateQuote', userPolicies.updateSalesQuote, false);
          break;
      case 'duplicate':
          $createQuote.hide();
          $updateCreateQuote.show();
          $updateQuote.addClass('secondary-action').removeClass('primary-action');
          adjustButtonAccessibility('updateQuote', userPolicies.updateSalesQuote, true);        
          $duplicateQuote.addClass('primary-action').removeClass('secondary-action');
          break;
  }
}


function setButtonsActionQuoteDisabledState(isDisabled) {
  

  adjustButtonAccessibility('createQuote', userPolicies.createSalesQuote, isDisabled);
  adjustButtonAccessibility('updateQuote', userPolicies.updateSalesQuote, isDisabled);
  adjustButtonAccessibility('deleteQuote', userPolicies.deleteSalesQuote, isDisabled);
  adjustButtonAccessibility('duplicateQuote', userPolicies.createSalesQuote, isDisabled);
  adjustButtonAccessibility('printPreview', userPolicies.documentPdfPreviewSalesQuote, isDisabled); 
  
  if ($(".updateQuote").attr('class').includes('secondary-action')) 
    adjustButtonAccessibility('updateQuote', userPolicies.updateSalesQuote, true);

}

$(document).ready(function() {
    //Trigger Quote Creation Process
    $(document).on('click', '.createQuote, .updateQuote, .duplicateQuote', function() {
        toggleQuotePreparationRows(true);
        if ($(this).attr('class').includes('updateQuote')) 
           dlgBoxYesNo("info", "Update this quote", "Do you confirm the update of this quote in Sage Active?", updateQuoteAction);  
        else
           dlgBoxYesNo("info","Create a quote","Do you confirm the creation of a new quote in Sage Active?",createQuoteAction);
    });

    function createQuoteAction(choice){
       createOrUpdateQuoteAction(choice,"create");
    }
    
    function updateQuoteAction(choice){
       createOrUpdateQuoteAction(choice,"update");
    }

    function createOrUpdateQuoteAction(choice,action){
      if (!choice){
        dlgBoxYesNoCancel();
        toggleQuotePreparationRows(false);
      }
      else{
        dlgBoxYesNoCancel();
        disableInterface();
        createOrUpdateSalesQuote(action);
      }
    }

    //delete Quote
    $(document).on('click', '.deleteQuote', function() {  
     dlgBoxYesNo("warning", "Delete the Current Quote", "Are you sure you want to delete this quote?<br><br>Please note that this deletion may be declined based on business rules.", deleteQuote);
     });

    function deleteQuote(choice){
      dlgBoxYesNoCancel();
      if (!choice) return;
      disableInterface();
  
      const values = {
        "values": {
          id: $('#customerLastQuotes').val()
        }
      };
      genericFetchPublicAPI(headerWithOrganization,qm.deleteSalesQuote,values, handleResponse);
      function handleResponse(data) {
        if (data && data.data && data.data.deleteSalesQuote) {
          getCustomerOrAllLastQuotes($('#customerId').val()).then(() => {
            $('#customerLastQuotes').val("").trigger("change");
          });
          readProductsFromTariffList();
          displayBusinessInfo(`Quote "No.${$('#customerLastQuotes').find(':selected').data().textForSearch}" has been successfully deleted`,true);
        }
        enableInterface();
      }
    }

    //preview pdf
    $(document).on('click', '.printPreview', function() {  
      dlgBoxYesNo("info", "PDF Preview", 
      `You are about to preview the document in its current state in Sage Active.
      <br><br>Please note, this preview does not include any unsaved changes.
      `
      , preparePreviewPdf);
    });

    function preparePreviewPdf(choice){
      dlgBoxYesNoCancel();
      if (!choice) return;

    const variables = {
        input:{
          type:"SALES_QUOTE",
          id: $('#customerLastQuotes').val()
        }
      };

      genericFetchPublicAPI(headerWithOrganization,qm.queryGetPdf,variables, handleResponse);
      function handleResponse(data) {
        if (data.data && data.data.documentPdfPreview) {
          const binaryPdf = data.data.documentPdfPreview.pdfPreview;
          previewPdf(binaryPdf);
          waitInfo(false);
        }
      }
          
    }

  });

  function previewPdf(base64String){
    const base64Pdf = `data:application/pdf;base64,${base64String}`;
    let pdfBlob = fetch(base64Pdf).then(res => res.blob());
    pdfBlob.then(blob => {
      const pdfUrl = URL.createObjectURL(blob);
      window.open(pdfUrl);
    });
  }

//Toggle Quote Preparation Visibility
function  toggleQuotePreparationRows(action) {
  var style=action ?"none":"table-row";
  $('#product-list tr').each(function() {
    if ($(this).attr('deleted')) return;
    const quantity = $(this).find('input[name="quantity"]').val();
    if (!quantity || quantity.trim() === '') {
          $(this).css("display", style);
      }
  });
};

 // Function to create or update a new sales quote
 function createOrUpdateSalesQuote(action) {
   let create=(action=="create");
   let quoteId;
   const currentQuoteId=$('#customerLastQuotes').val();
   const customerId = $('#customerId').val();
   const documentDate = (create) ? new Date().toISOString(): $('#documentDate').val();
   const discount = $('#discountPercentage').val() ? parseFloat($('#discountPercentage').val()) : 0;

   const lines = [];
 
   $("#product-list tr").each(function() {
    const row = $(this);
    if (create && row.attr("deleted")) return;
    const totalQuantity = parseFloat(row.find('input[name="quantity"]').val());
    if (totalQuantity && !isNaN(totalQuantity)) {

      const lineId = row.find('input[name="lineId"]').val();
      const productId = row.find('input[name="productId"]').val();
      const productName = row.find('input[name="name"]').val();
      const unitPrice = parseFloat(row.find('input[name="currentprice"]').val());
      const firstDiscount = parseFloat(row.find('input[name="discount"]').val());

      const newLine = {
        "productId": productId,
        "productName": productName,
        "totalQuantity": totalQuantity,
        "unitPrice": (!unitPrice)?0:unitPrice,
        "firstDiscount": (!firstDiscount)?0:firstDiscount,
      };
      if (!create && lineId) {
        newLine.id = lineId; 
        if (row.attr("deleted")) newLine.requestedAction= "DELETE";
      }
      lines.push(newLine);

    }
   });
  
   const mainAddressObject = {
    ...(create ? {} : { requestedAction: "MODIFY" }),
    ...($('#firstLine').val() !== '' && { firstLine: $('#firstLine').val() }),
    ...($('#secondLine').val() !== '' && { secondLine: $('#secondLine').val() }),
    ...($('#zipCode').val() !== '' && { zipCode: $('#zipCode').val() }),
    ...($('#city').val() !== '' && { city: $('#city').val() }),
    ...($('#isoAlpha2').val() !== '' && { countryIsoCodeAlpha2: $('#isoAlpha2').val() })
  };
  
  const values = {
    values: {
      documentDate,
      discount,
      lines,
      ...($('#contactName').val() !== '' && { contactName: $('#contactName').val() }),
      ...($('#socialName').val() !== '' && { socialName: $('#socialName').val() }),
      ...($('#contactPhone').val() !== '' && { contactPhone: $('#contactPhone').val() }),
      ...($('#contactEmail').val() !== '' && { contactEmail: $('#contactEmail').val() }),
    }
  };
  if (customerId=="LEAD" && Object.keys(mainAddressObject).length > 0) {
    values.values.mainAddress = [mainAddressObject];
  }

  if (create){
    if (customerId && customerId!="LEAD"){
        values.values.customerId = customerId;
    }
  }
  else{
    values.values.id = currentQuoteId;
  }

   genericFetchPublicAPI(headerWithOrganization, (create)?qm.createSalesQuote:qm.updateSalesQuote, values, handleResponse);

   function handleResponse(data) {
     enableInterface();
     if (data && data.data && (data.data.createSalesQuote || data.data.updateSalesQuote )) {
       quoteId = (data.data.createSalesQuote || data.data.updateSalesQuote).id ;
       dlgBoxYesNo("info",`Success! The quote has been ${(create)?"created":"updated"}.`,
       "Would you like to display this quote?<br><br>Choose No to start entering a new quote directly.",afterCreateQuote); 
     }
   }

   function afterCreateQuote(choice){
    if (!choice){
      dlgBoxYesNoCancel();
      resetCustomer();
    }
    else{
      dlgBoxYesNoCancel();
      disableInterface();
      $("#quoteId").val(""); // To force to read the quote even the id is the same when modification
      displayQuoteById(quoteId);
      getCustomerOrAllLastQuotes($('#customerId').val()).then(() => {
        $('#customerLastQuotes').val(quoteId).trigger("change");
        enableInterface();
      });
    }
  }
}


//Get quotes of the current customer or for all customers
function getCustomerOrAllLastQuotes(customerId,salesQuoteId) {
  if (customerId=="LEAD") customerId=null;

  qm.queryQuotesOfCurrentCustomerOrAll.noDataMessage = "[INFO]"+(customerId ? "No quotes found for this customer" : "No quotes created for customers or leads yet");

  return new Promise((resolve, reject) => {
    var whereClause=(customerId) ? `where: {customerId: {eq:"${customerId}"}}`:""; 

    genericFetchPublicAPI(headerWithOrganization, qm.queryQuotesOfCurrentCustomerOrAll, null,handleResponse,whereClause);
    function handleResponse(data) {
      if (data && data.data && data.data.salesQuotes && Array.isArray(data.data.salesQuotes.edges)) {
          const selectElement = $('#customerLastQuotes'); 
          selectElement.empty(); 
  
          if (data.data.salesQuotes.totalCount == 0) {
              selectElement.prop('disabled', true);
              resolve();
              return;
          }

          function prepareQuoteData(quote, customerId) {
            return {
                operationalNumber: quote.operationalNumber.trim(),
                formattedDate: formatDateMonthDay(quote.documentDate),
                socialName: quote.socialName ? quote.socialName.substring(0, 14).trim() : '',
                totalNet: quote.totalNet.toFixed(2),
                status: quote.status,
                showSocialName: !customerId
            };
          }
        

          var defaultOption = new Option("Select a Quote to View", "");
          defaultOption.disabled = true; 
          defaultOption.selected = true;
          selectElement.append(defaultOption);
  
          const salesQuotesList = data.data.salesQuotes.edges;         
          salesQuotesList.forEach(quoteNode => {
            var quote = quoteNode.node;
            var preparedData = prepareQuoteData(quote, customerId);
            var option = new Option("", quote.id);
            $(option).data('quoteData', preparedData);
            var textForSearch = `${preparedData.operationalNumber} ${preparedData.formattedDate} ${preparedData.socialName} ${preparedData.totalNet} ${preparedData.status}`;
            $(option).data('textForSearch', textForSearch);
            selectElement.append(option);
        });
          $('#customerLastQuotes').prop('disabled', false); 
          if (salesQuoteId) $("#customerLastQuotes").val(salesQuoteId).trigger("change");
          resolve();
      } 
    }
  
  })
}

// when select a quote from the list of quotes , display it
$(document).ready(function() {
  $('#customerLastQuotes').select2({
      templateResult: formatQuoteOption,
      templateSelection: formatQuoteOption,
      matcher: matchCustom
  });

  function formatQuoteOption(option) {
    if (!option.id || !$(option.element).data('quoteData')) {
        return option.text;
    }

    var quoteData = $(option.element).data('quoteData');
    var $option = $('<div class="customerLastQuotes-option"></div>');

    $option.append('<span class="column-number">No.' + quoteData.operationalNumber + '</span>');
    $option.append('<span class="column-date">' + quoteData.formattedDate + '</span>');
    if (quoteData.showSocialName) {
        $option.append('<span class="column-name">' + quoteData.socialName + '</span>');
    }  
    $option.append('<span class="column-total">' + quoteData.totalNet + '</span>');
    $option.append(`<span class="column-status status${quoteData.status}">${quoteData.status}</span>`);

    return $option;
  }
 
$('#customerLastQuotes').change(function() {
    var selectedQuote=$(this).val();
    if (!selectedQuote) return;
    displayQuoteById(selectedQuote);
    })
});

// display the quote by this id
function displayQuoteById(id){
    if (id==$('#quoteId').val()) return;
    disableInterface();
    deleteAllProductListRows();
    $("#templateProducts").val("").trigger('change');
    genericFetchPublicAPI(headerWithOrganization, qm.queryQuoteById, { id: id },handleResponse);
    function handleResponse(data){  
      if (data && data.data && data.data.salesQuotes && Array.isArray(data.data.salesQuotes.edges)) {
        var salesQuoteNode=data.data.salesQuotes.edges[0].node;
        if (salesQuoteNode.customerId!= $('#customerId').val()) {
          if (salesQuoteNode.socialName && salesQuoteNode.customerId && salesQuoteNode.customerId!="00000000-0000-0000-0000-000000000000"){
            $('#customerId').val("");
            $('#searchCustomer').val(salesQuoteNode.socialName);
            readCustomer(salesQuoteNode.socialName,id);
          }
          else{
            $('#customerId').val("LEAD");
            $('#discountPercentage').prop('readonly',false);
          }
        }
        readProducts(salesQuoteNode.lines);
        var labelCurrentQuote=`<div class="quoteNumberLabel">No.<strong>${salesQuoteNode.operationalNumber}</strong> - Date: <strong>${salesQuoteNode.documentDate.substring(0,10)}</strong></div><label class="status${salesQuoteNode.status}">${salesQuoteNode.status}</label>`;
        $(".quoteNumber").html(labelCurrentQuote);
        $('#discountPercentage').val((salesQuoteNode.discount==0)?"":salesQuoteNode.discount.toFixed(2));
        $('#documentDate').val(salesQuoteNode.documentDate);
        $('#quoteId').val(id);
        toggleButtonsActionQuote((salesQuoteNode.status=="Pending")?"updateOrDuplicate":"duplicate");
        var socialName=(salesQuoteNode.socialName)?salesQuoteNode.socialName:"UNKNOWN";
        var contactName=(salesQuoteNode.contactName)?salesQuoteNode.contactName:socialName;
        $('#contactName').val(contactName).trigger('input');
        $('#socialName').val(socialName);
        $('#contactPhone').val(salesQuoteNode.contactPhone);
        $('#contactEmail').val(salesQuoteNode.contactEmail);
        var mainAddress = Array.isArray(salesQuoteNode.mainAddress) ? salesQuoteNode.mainAddress[0] : salesQuoteNode.mainAddress; // break of compatibility with data loaders
        $('#firstLine').val(mainAddress.firstLine);
        $('#secondLine').val(mainAddress.secondLine);
        $('#zipCode').val(mainAddress.zipCode);
        $('#city').val(mainAddress.city);
        $('#autoCompleteCountry').val(salesQuoteNode.countryAcronym).trigger('change');
        
        fillInCustomerInfoFromLead();
 
      }
    }
}

