
// Get list of tariffs and store in a dictionary in memory
var tariffsDictionary = {};
function readTariffs() {
  var qmQueryTariffs={...qm.queryTariffs};
  qmQueryTariffs.noDataMessage="[INFO]No product list template found in the price lists, two demonstration templates are emulated.";
  return new Promise((resolve, reject) => {
    genericFetchPublicAPI(headerWithOrganization, qmQueryTariffs, null, handleResponse);

    function handleResponse(data) {
      if (data && data.data && data.data.salesTariffs && Array.isArray(data.data.salesTariffs.edges)) {
        const tariffList = data.data.salesTariffs.edges;
        tariffsDictionary={};
        tariffList.forEach(tariff => {
          tariffsDictionary[tariff.node.id] = {
            code: tariff.node.code,
            name: tariff.node.name,
            template: !tariff.node.enabled && !tariff.node.startDate && !tariff.node.endDate
          };
        });
        const hasTemplate = Object.values(tariffsDictionary).some(tariff => tariff.template);
        if (!hasTemplate) {
          tariffsDictionary["FAKE_BESTSALES"] = {
            code: "BESTSALES",
            name: "Best Sales",
            template: true
          };
          tariffsDictionary["FAKE_STANDARD"] = { 
            code: "FAVORITES",
            name: "Favorites products",
            template: true
          };
        }
        resolve(); 
      }
    }
  });
}

// Define the dropdown of templates of products (= tariffs where template is true)
function populateTariffDropdown() {
    const selectElement = $('#templateProducts'); 
    selectElement.empty(); 
    var option = new Option("Select a List of products", ""); 
    option.disabled = true; 
    selectElement.append(option); 
    selectElement.append($('<option></option>').attr('value', 'ALLPRODUCTS').text('All products'));
  
    Object.keys(tariffsDictionary).forEach(key => {
      const tariff = tariffsDictionary[key];
      if (tariff.template) {
        selectElement.append($('<option></option>').attr('value', tariff.code).text(tariff.name));
      }
    });
  }
  
  $(document).ready(function() {

    $('#templateProducts').select2({
      minimumResultsForSearch: -1,
    });

    $('#templateProducts').change(function() {
      var selectedList=$(this).val();
      if (!selectedList) return;
      disableInterface();
      deleteAllProductListRows();
      $("#customerLastQuotes").val("").trigger("change");
      $(".quoteNumber").html("New quote");
      $('#discountPercentage').val($('#customerDiscount').val());
      readProductsFromTariffList(selectedList);
    });
  });

  
//Get tariff Name with cache
function getTariffName(id, callback) {
  if (!id) 
  callback("Standard");
else
{
    //Read tariffName in cache
    if (tariffsDictionary[id]) {
      callback(tariffsDictionary[id].name);
    }
    else {
        // dynamically get tariff by Id 
        genericFetchPublicAPI(headerWithOrganization, qm.queryTariffById, {id:id},handleResponse);
        function handleResponse(data){
          if (data && data.data && data.data.salesTariffs) {
            const tariff = data.data.salesTariffs.edges[0].node;
            callback(tariff.name);
          }
        }
     }
  }
}

//Get discount Name without cache
function getDiscountName(id, callback) {
  if (!id) 
    callback("Standard");
  else
  {
    genericFetchPublicAPI(headerWithOrganization, qm.queryDiscountById, {id:id},handleResponse);
    function handleResponse(data){
      if (data && data.data && data.data.salesDiscounts) {
        callback(data.data.salesDiscounts.edges[0].node.name);
      }
      else
        callback("unknown");
    }
  }
}
