var userPolicies= {};

//Fetches the access policies for the current user using the specified policies used by the application.
function checkUserAccessPolicies() {
  return new Promise((resolve, reject) => {
    const query = qm.queryUserAccessPolicyCheck; 
    var variables = { actions: getListOfPolicies() };
    
    genericFetchPublicAPI(headerWithOrganization, query, variables, handleResponse);

    function handleResponse(data) {
      if (data && data.data && data.data.userAccessPolicyCheck) {
        userPolicies = createPolicyAuthorizationMap(data);
        resolve();
      } else {
        reject();
      }
    }
  });
}

//Get all policies from queries_mutations.js (remove duplicates and null values)
function getListOfPolicies() {
  return [...new Set(Object.values(qm).filter(item => item.policy !== null && item.policy !=="none").map(item => item.policy))];
}

//Creates a map of policies and their authorization status
function createPolicyAuthorizationMap(userAccessData) {
  const policyAuthorizationMap = {};
  userAccessData.data.userAccessPolicyCheck.forEach(policyCheck => {
    policyAuthorizationMap[policyCheck.action] = policyCheck.isAllowed;
  });
  return policyAuthorizationMap;
}

//Adjusts the accessibility of a button based on the user's policy and the business rules
function adjustButtonAccessibility  (buttonId, policyAllowed, forceDisable) {
  if (policyAllowed=== null) 
    policyAllowed=true;
  else if (policyAllowed=== undefined) {
    setTimeout(() => {
      waitInfo(false);
      displayBusinessError("UNKNOWNUSERPOLICY", "adjustButtonAccessibility : " + buttonId);
    }, 1000);
  }

  const $button = $('#' + buttonId);
  const shouldDisable = !policyAllowed || forceDisable;
  $button.prop('disabled', shouldDisable);
  if (!policyAllowed) {
    $button.addClass('unauthorized');
  } else {
    $button.removeClass('unauthorized');
  }
}

// Check user's application access
function checkUserAppAccess() {
  if (!userPolicies.salesQuotes) {
    waitInfo(false);
    deleteAllProductListRows();
    displayBusinessError("APPFORBIDDEN");
    return false;
  } else 
    return true;
}
